/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** 
** Author: Didier Lemoine		Created: 20-07-93 
** 
** Module: db_utils 
** 
** Purpose: 	
** This file contains various facilities functions used to implement
** the DB Module.
** This file also contains DB_Handle manipulation functions.
** 
** Module Prefix: DB_ 
** 
** Last Update:		$Author: hewett $
** Update Date:		$Date: 1997/10/06 13:48:03 $
** Source File:		$Source: /share/dicom/cvs-depot/dcmtk/imagectn/libsrc/dbutils.cc,v $
** CVS/RCS Revision:	$Revision: 1.14 $
** Status:		$State: Exp $
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#include <errno.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#include <ctype.h>
#include <string.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
END_EXTERN_C

#include "imagedb.h"
#include "dbpriv.h"
#include "dcdatset.h"
#include "dcdeftag.h"
#include "diutil.h"

/*************************
 *
 *	Static Data
 *
 *************************/

/**** The TbFindAttr table contains the description of tags (keys) supported
 **** by the DB Module.
 **** Tags described here have to be present in the Index Record file. 
 **** The order is unsignificant.
 ****
 **** Each element of this table is described by
 ****		The tag value
 ****		The level of this tag (from patient to image)
 ****		The Key Type (only UNIQUE_KEY values is used)
 ****		The key matching type, specifiing which type of
 ****			matching should be performed. The OTHER_CLASS
 ****			value specifies that only strict comparison is applied.
 ****
 **** This table and the IndexRecord structure should contain at least
 **** all Unique and Required keys.
 ***/

static DB_FindAttr TbFindAttr [] = {
	DB_FindAttr( DCM_PatientBirthDate ,		PATIENT_LEVEL,	OPTIONAL_KEY,	DATE_CLASS	),
	DB_FindAttr( DCM_PatientSex,			PATIENT_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_PatientName,			PATIENT_LEVEL, 	REQUIRED_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_PatientID,			PATIENT_LEVEL, 	UNIQUE_KEY,		STRING_CLASS	),
	DB_FindAttr( DCM_PatientBirthTime,		PATIENT_LEVEL, 	OPTIONAL_KEY,	TIME_CLASS	),
	DB_FindAttr( DCM_OtherPatientID,			PATIENT_LEVEL, 	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_OtherPatientNames,		PATIENT_LEVEL, 	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_EthnicGroup,			PATIENT_LEVEL, 	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_PatientComments,			PATIENT_LEVEL, 	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_NumberOfPatientRelatedStudies,	PATIENT_LEVEL, 	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_NumberOfPatientRelatedSeries,	PATIENT_LEVEL, 	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_NumberOfPatientRelatedImages,	PATIENT_LEVEL, 	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_StudyDate,			STUDY_LEVEL,	REQUIRED_KEY,	DATE_CLASS	),
	DB_FindAttr( DCM_StudyTime,			STUDY_LEVEL,	REQUIRED_KEY,	TIME_CLASS	),
	DB_FindAttr( DCM_StudyID,				STUDY_LEVEL,	REQUIRED_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_AccessionNumber,			STUDY_LEVEL,	REQUIRED_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_ReferringPhysiciansName,		STUDY_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_StudyDescription,			STUDY_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_NameOfPhysiciansReadingStudy,		STUDY_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_StudyInstanceUID,			STUDY_LEVEL,	UNIQUE_KEY,		UID_CLASS	),
	DB_FindAttr( DCM_OtherStudyNumbers,		STUDY_LEVEL,	OPTIONAL_KEY,	OTHER_CLASS	),
	DB_FindAttr( DCM_AdmittingDiagnosisDescription,	STUDY_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_PatientAge,			STUDY_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_PatientSize,			STUDY_LEVEL,	OPTIONAL_KEY,	OTHER_CLASS	),
	DB_FindAttr( DCM_PatientWeight,			STUDY_LEVEL,	OPTIONAL_KEY,	OTHER_CLASS	),
	DB_FindAttr( DCM_Occupation,			STUDY_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_AdditionalPatientHistory,		STUDY_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_NumberOfStudyRelatedSeries,	STUDY_LEVEL,	OPTIONAL_KEY,	OTHER_CLASS	),
	DB_FindAttr( DCM_NumberOfStudyRelatedImages,	STUDY_LEVEL,	OPTIONAL_KEY,	OTHER_CLASS	),
	DB_FindAttr( DCM_SeriesNumber,			SERIE_LEVEL,	REQUIRED_KEY,	OTHER_CLASS	),
	DB_FindAttr( DCM_SeriesInstanceUID,		SERIE_LEVEL,	UNIQUE_KEY,		UID_CLASS	),
	DB_FindAttr( DCM_Modality,				SERIE_LEVEL,	OPTIONAL_KEY,	STRING_CLASS	),
	DB_FindAttr( DCM_ImageNumber,			IMAGE_LEVEL,	REQUIRED_KEY,	OTHER_CLASS	),
	DB_FindAttr( DCM_SOPInstanceUID,			IMAGE_LEVEL,	UNIQUE_KEY,		UID_CLASS	)
  } ;

/**** The NbFindAttr variable contains the length of the TbFindAttr table
 ***/

static int NbFindAttr = ((sizeof (TbFindAttr)) / (sizeof (TbFindAttr [0]))) ;


int DB_debugLevel = 0;

void DB_setDebugLevel(int debugLevel)
{
    DB_debugLevel = debugLevel;
}

int DB_getDebugLevel()
{
    return DB_debugLevel;
}

void DB_debug(int level, const char* format, ...)
{
    va_list ap;

    if (level <= DB_debugLevel) {
	fprintf(stderr, "DB:");
	va_start(ap, format);
	vfprintf(stderr, format, ap);
	va_end(ap);
    }
}

void DB_printDataset(DcmDataset *ds)
{
    ds->print();
}

static OFBool doCheckFindIdentifier = OFFalse;
static OFBool doCheckMoveIdentifier = OFFalse;

void DB_setIdentifierChecking(OFBool checkFind, OFBool checkMove)
{
    doCheckFindIdentifier = checkFind;
    doCheckMoveIdentifier = checkMove;
}

OFBool DB_doCheckFindIdentifier()
{
    return doCheckFindIdentifier;
}

OFBool DB_doCheckMoveIdentifier()
{
    return doCheckMoveIdentifier;
}


CONDITION 
DB_lock(DB_Private_Handle *phandle, OFBool exclusive)
{
    int lockmode;
	
    if (exclusive) {
	lockmode = LOCK_EX;	/* exclusive lock */
    } else {
	lockmode = LOCK_SH;	/* shared lock */
    }
    if (flock(phandle->pidx, lockmode) < 0) {
	perror("flock");
	return DB_ERROR;
    }
    return DB_NORMAL;
}

CONDITION 
DB_unlock(DB_Private_Handle *phandle)
{
    if (flock(phandle->pidx, LOCK_UN) < 0) {
	perror("flock");
	return DB_ERROR;
    }
    return DB_NORMAL;
}



/***********************
 *	Creates a handle
 */

CONDITION 
DB_createHandle(
		const char 	*storageArea,
		long 	maxStudiesPerStorageArea,
		long 	maxBytesPerStudy,
		DB_Handle 	**handle)
{
    DB_Private_Handle *phandle ;

    phandle = (DB_Private_Handle *) malloc ( sizeof(DB_Private_Handle) ) ;

#ifdef DEBUG	
    DB_debug(1, "DB_createHandle () : Handle created for %s\n",storageArea) ;
    DB_debug(1, "                     maxStudiesPerStorageArea: %ld maxBytesPerStudy: %ld\n",
	    maxStudiesPerStorageArea, maxBytesPerStudy) ;
#endif

    if (maxStudiesPerStorageArea > DB_UpperMaxStudies) {
	fprintf(stderr, "WARING: maxStudiesPerStorageArea too large\n");
	fprintf(stderr, "        setting to %d\n", DB_UpperMaxStudies);
	maxStudiesPerStorageArea = DB_UpperMaxStudies;
    }
    if (maxStudiesPerStorageArea < 0) {
	maxStudiesPerStorageArea = DB_UpperMaxStudies;
    }
    if (maxBytesPerStudy < 0 || maxBytesPerStudy > DB_UpperMaxBytesPerStudy) {
	maxBytesPerStudy = DB_UpperMaxBytesPerStudy;
    }

    if (phandle) {
	sprintf (phandle -> storageArea,"%s", storageArea) ;
	sprintf (phandle -> indexFilename,"%s%c%s", storageArea, PATH_SEPARATOR, DBINDEXFILE) ;

	/* create index file if it does not already exist */
	FILE* f = fopen(phandle->indexFilename, "ab");
	if (f == NULL) {
	    perror(phandle->indexFilename);
	    return DB_ERROR;
	}
	fclose(f);

	/* open fd of index file */
#ifdef O_BINARY
	phandle -> pidx = open(phandle -> indexFilename, O_RDWR | O_BINARY ) ;
#else
	phandle -> pidx = open(phandle -> indexFilename, O_RDWR ) ;
#endif
	if ( phandle -> pidx == (-1) ) 
	    return ( DB_CANNOTCREATEHANDLE_CannotOpenIdx ) ;
	else {
	    phandle -> idxCounter = -1  ;
	    phandle -> findRequestList = NULL ;
	    phandle -> findResponseList = NULL  ;
	    phandle -> maxBytesPerStudy = maxBytesPerStudy ;
	    phandle -> maxStudiesAllowed = maxStudiesPerStorageArea ;
	    phandle -> uidList = NULL  ;
	    *handle = (DB_Handle *)phandle ;
	    return ( DB_NORMAL ) ;
	}
    }
    else
			 
	return ( DB_CANNOTCREATEHANDLE_CannotAllocate ) ;

}

/***********************
 *	Destroys a handle
 */

CONDITION 
DB_destroyHandle(DB_Handle **handle) 
{
    DB_Private_Handle *phandle ;
    int closeresult ;

    phandle = (DB_Private_Handle *) (*handle) ;
    if (phandle)
    {
      DB_unlock(phandle);
      closeresult = close( phandle -> pidx) ;

      /* Free lists */	
      DB_FreeElementList (phandle -> findRequestList) ;
      DB_FreeElementList (phandle -> findResponseList) ;
      DB_FreeUidList (phandle -> uidList) ;

      free ( (char *)(phandle) ) ;
      if ( closeresult )
  	return ( DB_CANNOTDESTROYHANDLE_CannotCloseIdx ) ;
      else
  	return ( DB_NORMAL ) ;
    } else {
      return ( DB_CANNOTDESTROYHANDLE_CannotCloseIdx );	
    }
}

/**********************************
 *	Provides a storage filename
 */

CONDITION 
DB_makeNewStoreFileName(
		DB_Handle  	*handle,
		const char 	*SOPClassUID,
		const char 	* /*SOPInstanceUID*/,
		char 		*newImageFileName)
{

    DB_Private_Handle *phandle ;

    phandle = (DB_Private_Handle *) handle ;

    /*
     * Use descriptive filename prefixes for the images. 
     * This makes it much easier during debugging to see
     * what is going on.  The first three character are
     * a symbolic representation of the SOPClassUID.
     */
    {
	char prefix[12];
	const char *m;
	char *t;
	    
	m = DU_sopClassToModality(SOPClassUID);
	if (m == NULL) {
	    m = "XX";   
	}
	sprintf(prefix, "%s.", m);
	t = tempnam(phandle -> storageArea, prefix);
	if (t != NULL) {
	    strcpy(newImageFileName, t);
	    free(t);	/* free the string malloced by tempnam */
	} else {
	    fprintf(stderr, "DB_makeNewStoreFileName: out of memory\n");
	    return DB_ERROR;
	}
    }

    return DB_NORMAL ;

}


/*******************
 *	Free an element List 
 */

CONDITION 
DB_FreeUidList (DB_UidList *list) 
{
    CONDITION cond  ;

    if (list == NULL)
	return DB_NORMAL ;

    cond = DB_FreeUidList (list -> next) ;
    if (list -> patient)
	free (list -> patient) ;
    if (list -> study)
	free (list -> study) ;
    if (list -> serie)
	free (list -> serie) ;
    if (list -> image)
	free (list -> image) ;
    free (list) ;
    return (cond) ;
}


/*******************
 *	Free a UID List 
 */

CONDITION 
DB_FreeElementList (DB_ElementList *list) 
{
    CONDITION cond ;

    if (list == NULL)
	return DB_NORMAL ;

    cond = DB_FreeElementList (list -> next) ;
    if (list->elem.PValueField != NULL) {
	free ((char *) list -> elem. PValueField) ;
    }
    free (list) ;
    return (cond) ;
}


/*******************
 *	Matches two strings 
 */

int
DB_StringUnify  (char *pmod, char *pstr)
{
    int uni ;
 
 
    if (*pmod == '\0')
	return (*pstr == '\0') ;
 
    if (  *pmod == *pstr
	  || (*pmod == '?' && *pstr != '\0')
	)
	return (DB_StringUnify (pmod + 1, pstr + 1)) ;
 
    if (*pmod == '*') {
	if ( *(pmod + 1) == '\0' )
	    return (OFTrue) ;
	while (  ( (uni = DB_StringUnify (pmod + 1, pstr)) == OFFalse )
		 && (*pstr != '\0')
	    )
	    pstr++ ;
	return (uni) ;
    }
    else if (*pmod != *pstr)
	return (OFFalse) ;
    return OFFalse;
}
 
/*******************
 *	Is the specified tag supported 
 */

int			
DB_TagSupported (DcmTagKey tag) 
{
    int i ;

    for (i = 0 ; i < NbFindAttr ; i++)
	if (TbFindAttr[i]. tag == tag)
	    return (OFTrue) ;

    return (OFFalse) ;

}

 
/*******************
 *	Get UID tag of a specified level 
 */

CONDITION			
DB_GetUIDTag (DB_LEVEL level, DcmTagKey *tag) 
{
    int i ;

    for (i = 0 ; i < NbFindAttr ; i++)
	if ((TbFindAttr[i]. level == level) && (TbFindAttr[i]. keyAttr == UNIQUE_KEY))
	    break ;

    if (i < NbFindAttr) {
	*tag = TbFindAttr[i].tag ;
	return (DB_NORMAL) ;
    }
    else
	return (DB_ERROR) ;

}

/*******************
 *	Get tag level of a specified tag 
 */

CONDITION			
DB_GetTagLevel (DcmTagKey tag, DB_LEVEL *level)
{
    int i ;

    for (i = 0 ; i < NbFindAttr ; i++)
	if (TbFindAttr[i]. tag == tag)
	    break ;

    if (i < NbFindAttr) {
	*level = TbFindAttr[i]. level ;
	return (DB_NORMAL) ;
    }
    else
	return (DB_ERROR) ;
}

/*******************
 *	Get tag key attribute of a specified tag 
 */

CONDITION			
DB_GetTagKeyAttr (DcmTagKey tag, DB_KEY_TYPE *keyAttr)
{
    int i ;

    for (i = 0 ; i < NbFindAttr ; i++)
	if (TbFindAttr[i]. tag == tag)
	    break ;

    if (i < NbFindAttr) {
	*keyAttr = TbFindAttr[i]. keyAttr ;
	return (DB_NORMAL) ;
    }
    else
	return (DB_ERROR) ;
}

/*******************
 *	Get tag key attribute of a specified tag 
 */

CONDITION			
DB_GetTagKeyClass (DcmTagKey tag, DB_KEY_CLASS *keyAttr)
{
    int i ;

    for (i = 0 ; i < NbFindAttr ; i++)
	if (TbFindAttr[i]. tag == tag)
	    break ;

    if (i < NbFindAttr) {
	*keyAttr = TbFindAttr[i]. keyClass ;
	return (DB_NORMAL) ;
    }
    else
	return (DB_ERROR) ;
}


/*******************
 *	Remove spaces in a string  
 */

void			
DB_RemoveSpaces (char *string)
{
    char *pc1, *pc2 ;

    for (pc1 = pc2 = string ; *pc2 ; pc2++) {
	if (*pc2 != ' ') {
	    *pc1 = *pc2 ;
	    pc1++ ;
	}
    }
    *pc1 = '\0' ;
}

/*******************
 *	Remove leading and trailing spaces in a string  
 */

void
DB_RemoveEnclosingSpaces (char *string)
{
    char *pc1, *pc2 ;

    /** Find in pc2 the first non space character
    ** If not found, string is empty
    */

    for (pc2 = string ; (*pc2 != '\0') && (*pc2 == ' ') ; pc2++) ;
    if (*pc2 == '\0') {
	string [0] = '\0' ;
	return ;
    }

    /** Shift the string if necessary
     */
 
    if (pc2 != string) {
	for (pc1 = string ; *pc2  ; pc1++, pc2++)
	    *pc1 = *pc2 ;
	*pc1 = '\0' ;
    }

    /** Ship trailing spaces
     */

    for (pc2 = string + strlen (string) - 1 ; *pc2 == ' ' ; pc2--) ;
    pc2++ ;
    *pc2 = '\0' ;
}


/*******************
 *	Convert a date YYYYMMDD in a long  
 */

long
DB_DateToLong (char *date)
{
    char year [5] ;
    char month[3] ;
    char day  [3] ;

    strncpy (year, date, 4) ;
    year [4] = '\0' ;
    strncpy (month, date + 4, 2) ;
    month [2] = '\0';
    strncpy (day, date + 6, 2);
    day [2] = '\0';

    return ((atol(year) * 10000) + (atol(month) * 100) + atol(day));
}


/*******************
 *	Convert a time in a double
 */

double
DB_TimeToDouble (char *time)
{
    char t [20] ;
    char tmp [3] ;

    double result = 0. ;
    char *pc ;

    /*** Get fractionnal part if exists
    **/

    strcpy (t, time) ;
    if ((pc = strchr (t, '.')) != NULL) {
	double f ;

	*pc = '\0' ;
	for (pc++, f = 1. ; (*pc) && (isdigit (*pc)) ; pc++) {
	    f /= 10. ;
	    result += (*pc - '0') * f ;
	}
    }

    /*** Add default values (mm ss) if necessary
    **/

    strcat (t, "0000") ;
    t [6] = '\0' ;

    /*** Get Hours, Minutes and Seconds
    **/

    strncpy (tmp, t, 2) ;
    tmp [3] = '\0' ;
    result += 3600. * atof (tmp) ;

    strncpy (tmp, t + 2, 2) ;
    tmp [3] = '\0' ;
    result += 60. * atof (tmp) ;

    strncpy (tmp, t + 4, 2) ;
    tmp [3] = '\0' ;
    result += atof (tmp) ;

    return result ;
}



/***********************
 *	Duplicate a dicom element
 *	dst space is supposed provided by the caller
 */

void 
DB_DuplicateElement (DB_SmallDcmElmt *src, DB_SmallDcmElmt *dst) 
{
    bzero( (char*)dst, sizeof (DB_SmallDcmElmt)) ;
    dst -> XTag = src -> XTag ;
    dst -> ValueLength = src -> ValueLength ;

    if (src -> ValueLength == 0)
	dst -> PValueField = NULL ;
    else {
	dst -> PValueField = (char *)malloc ((int) src -> ValueLength+1) ;
	bzero(dst->PValueField, (size_t)(src->ValueLength+1));
	if (dst->PValueField != NULL) {
	    memcpy (dst -> PValueField,  src -> PValueField, 
		    (size_t) src -> ValueLength) ;
	} else {
	    fprintf(stderr, "DB_DuplicateElement: out of memory\n");
	}
    }
}


/***********************
 *	Compare two ImagesofStudyArray elements
 */

int DB_Compare(ImagesofStudyArray *e1, ImagesofStudyArray *e2) 
{
    if ( e1 -> RecordedDate > e2 -> RecordedDate ) 
	return (1) ;
    else
	if ( e1 -> RecordedDate == e2 -> RecordedDate ) 
	    return (0) ;
	else
	    return (-1) ;

}




/*
** CVS Log
** $Log: dbutils.cc,v $
** Revision 1.14  1997/10/06 13:48:03  hewett
** Minor correction to imagectn's index file code to use the changed
** attribute names from dcdeftag.h
**
** Revision 1.13  1997/09/18 08:11:05  meichel
** Many minor type conflicts (e.g. long passed as int) solved.
**
** Revision 1.12  1997/08/06 12:20:20  andreas
** - Using Windows NT with Visual C++ 4.x the standard open mode for files
**   is TEXT with conversions. For binary files (image files, imagectn database
**   index) this must be changed (e.g. fopen(filename, "...b"); or
**   open(filename, ..... |O_BINARY);)
**
** Revision 1.11  1997/08/05 07:40:46  andreas
** Change definition of path to database index now using consistently
** the defines PATH_SEPARATOR and DBINDEXFILE
**
** Revision 1.10  1997/07/21 08:59:59  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.9  1997/06/26 12:52:02  andreas
** - Changed names for enumeration values in DB_KEY_TYPE since the value
**   OPTIONAL was predefined for Windows 95/NT
**
** Revision 1.8  1996/09/27 08:46:51  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C.
**
** Revision 1.7  1996/06/10 13:44:01  meichel
** Corrected error producing core dump with incorrect configrc file:
** imagectn tried to unlock an unsuccessfully allocated lockfile.
**
** Revision 1.6  1996/05/30 17:45:18  hewett
** Modified the definition of a static array of structs which was causing
** some C++ compilers problems.
**
** Revision 1.5  1996/04/29 15:16:06  hewett
** Removed unused DB_GetUSValue().
**
** Revision 1.4  1996/04/27 12:55:18  hewett
** Removed cause of warnings when compiled with "c++ -O -g -Wall" under
** Solaris 2.4.  Mostly due to uninitialized variables.
**
** Revision 1.3  1996/04/25 16:35:14  hewett
** Added char* parameter casts for bzero() calls.
**
** Revision 1.2  1996/04/22 11:23:44  hewett
** Added a function to print a DcmDataset (useful for debugging).
**
** Revision 1.1.1.1  1996/03/28 19:25:00  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
